; Copyright (C) 1994 by Dan Liddell and Robert Armstrong.
; Copyright (C) 2004 by Spare Time Gizmos.
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful, but
; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
; ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
; Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, visit the website of the Free
; Software Foundation, Inc., www.gnu.org.

;clock layout for analog. twelve hour time, of course.
;  the seconds are the same (yay)
;  minutes are a little more trouble. 40 LED's to display 60 minutes,
;      and it should sync up Exactly at 15, 30, 45 and 0 minutes
;  exactly 24 led's to display 12 hours 
;  no way to set this clock? Well, we could use the same logic as we
;      use for the digital clock.
;  silly cleverness: make the circles of minutes and seconds bang and
;      rattle around, kind of randomly, just for a little more visual
;      interest
;
;0 1 2 3 4 5 6 7 8 9 101112131415
;# # # # # # # # # # # # # # # # 0
;#                             # 1
;#     # # # # # # # # # #     # 2
;#   #                     #   # 3
;#   #     # # # # # #     #   # 4
;#   #   #             #   #   # 5
;#   #   #             #   #   # 6
;#   #   #             #   #   # 7
;#   #   #             #   #   # 8
;#   #   #             #   #   # 9
;#   #   #             #   #   # 10
;#   #     # # # # # #     #   # 11
;#   #                     #   # 12
;#     # # # # # # # # # #     # 13
;#                             # 14
;# # # # # # # # # # # # # # # # 15
;0 1 2 3 4 5 6 7 8 9 101112131415
;
; set up the screen for the analog clock
AINITCLOCK:
    call CLEARSCREEN

    mov r0,#7
    mov r1,#7
    call setxy

    mov r0,#7
    mov r1,#8
    call setxy

    mov r0,#8
    mov r1,#7
    call setxy

    mov r0,#8
    mov r1,#8
    call setxy

;    call DISPLAY_UPDATE

    jnb FLAG_DOFADEUP,ACLOCKINIT2
    clr FLAG_DOFADEUP
    call FADEUP
ACLOCKINIT2:    
    jb FLAG_TIMEISSET,ACLOCKINIT2A

    mov DSECONDS,#0	; if time is not set, then it will not get past 0
    mov DMINUTES,#0
    mov	DHOURS,#0

    sjmp ACLOCKINIT2B
ACLOCKINIT2A:
    mov DSECONDS,SECONDS

    mov a,SECONDS
    mov b,#30
    div ab
    rrc a
    mov a,MINUTES
    rlc a
    mov DMINUTES,a

    mov a,MINUTES
    mov b,#30
    div ab
    rrc a
    mov a,HOURS
    rlc a
    mov DHOURS,a

ACLOCKINIT2B:
    call WRITESECONDS
    mov a,DMINUTES
    call WRITEAMINUTES
    mov a,DHOURS
    call WRITEAHOURS

    call DISPLAY_UPDATE
    clr FLAG_DISPLAYCLOCK
    ret


ACLOCKUNTIL:
    clr FLAG_DIGITALCLOCK
    call AINITCLOCK
ACLOCKSECONDS:
    jb FLAG_TIMEISSET,ACLOCKSECONDSA ; don't do anything else, until clock set
    sjmp ACLOCKINPUT
ACLOCKSECONDSA:
    mov a,SECONDS
    cjne a,DSECONDS,ACLOCKDOSECONDS
ACLOCKMINUTES:
    mov a,SECONDS
    mov b,#30
    div ab
    rrc a
    mov a,MINUTES
    rlc a		; we have the number of 30 second time units so far
    push acc		; see if we should even check to see if it is the 
    mov b,#3		; write time to write another minute tic
    div ab
    mov a,b
    jnz ACLOCKMINUTES2	; nope. don't even check.
    pop acc
    cjne a,DMINUTES,ACLOCKDOMINUTES	; ok. maybe
    sjmp ACLOCKHOURS
ACLOCKMINUTES2:
    pop acc
ACLOCKHOURS:
    mov a,MINUTES
    mov b,#30
    div ab
    rrc a
    mov a,HOURS
    rlc a
    cjne a,DHOURS,ACLOCKDOHOURS
ACLOCKDISPLAY:
    jnb FLAG_DISPLAYCLOCK,ACLOCKINPUT
    call DISPLAY_UPDATE
    clr FLAG_DISPLAYCLOCK
ACLOCKINPUT:
    call GETKEY
    jz ACLOCKSECONDS
; handle any user input here
    call SETCLOCK
    call AINITCLOCK
    sjmp ACLOCKSECONDS
ACLOCKEXIT:
    clr a
    ret

ACLOCKDOSECONDS:
    mov DSECONDS,a
    mov r2,a
    mov r3,#0
    call DISPLAYSECONDS
    setb FLAG_DISPLAYCLOCK
    sjmp ACLOCKMINUTES
ACLOCKDOMINUTES:
    mov DMINUTES,a
    call DISPLAYAMINUTES
    setb FLAG_DISPLAYCLOCK
    sjmp ACLOCKHOURS
ACLOCKDOHOURS:
    mov DHOURS,a
    call DISPLAYAHOURS
    setb FLAG_DISPLAYCLOCK
    sjmp ACLOCKDISPLAY

; writeahours - starting from a blank screen, we set it up
;     to display the current correct hours
; a - the hour to write
WRITEAHOURS:
    mov r2,a
    jz HWRITEAM2
HWRITEAM:
    mov a,r2
    call DISPLAYAHOURS
    djnz r2,HWRITEAM
HWRITEAM2:
    mov a,r2
    call DISPLAYAHOURS
    ret

;0 1 2 3 4 5 6 7 
;  # # # # # #   0
;#             # 1
;#             # 2
;#             # 3
;#             # 4
;#             # 5
;#             # 6
;  # # # # # #   7
;0 1 2 3 4 5 6 7 
;
; displayahours - display analog hours. We assume that the 
;     screen has the current correct time on it
; a - the hour to display (twenty four hour time, at first)
; should be called every half hour, with a fixed point binary number,
;  with one digit behind the binary point.
; 0 1 2 3 4  5  6  7  8  9 10 11 12 13 14 15 16 17 18 19 20 21 22 23
; 0 2 4 6 8 10 12 14 16 18 20 22  0  2  4  6  8 10 12 14 16 18 20 22
DISPLAYAHOURS:
    cjne a,#24,$+3		; convert to 12 hour time.
    jc DAH1			; or maybe not.
    clr c
    subb a,#24
DAH1:
    mov r1,a			; first, convert to "LED" coordinates
    add a,r1			; double the accumulator

    clr c			; halve it, to get rid of the useless zero
    rrc a

    cjne a,#3,$+3
    jc HTOPRIGHT
    cjne a,#9,$+3
    jc HRIGHT
    cjne a,#15,$+3
    jc HBOTTOM
    cjne a,#21,$+3
    jc HLEFT
    sjmp HTOPLEFT
; perform the processing for the coordinates 
HTOPLEFT:
    clr c
    subb a,#21			; make the led coordinate 0 based

    mov r0,#0			; get the y ...
    mov r1,#1			;    ...and x coordinates of the first led

    add a,r1			; modify them to reflect the desired led
    mov r1,a
    sjmp HDISPMIN
HLEFT:
    clr c
    subb a,#15

    mov r0,#6
    mov r1,#0

    xch a,r0
    subb a,r0
    mov r0,a
    sjmp HDISPMIN
HBOTTOM:
    clr c
    subb a,#9

    mov r0,#7
    mov r1,#6

    xch a,r1
    subb a,r1
    mov r1,a
    sjmp HDISPMIN
HRIGHT:
    clr c
    subb a,#3

    mov r0,#1
    mov r1,#7

    add a,r0
    mov r0,a
    sjmp HDISPMIN
HTOPRIGHT:
    mov r0,#0			; no normalization necessary
    mov r1,#4

    add a,r1
    mov r1,a
    sjmp HDISPMIN
; add the absolute screen coordinates to the displacements that we have 
; just obtained
HDISPMIN:
    mov a,r0	; add the displacements (5,5)
    add a,#4
    mov r0,a

    mov a,r1
    add a,#4
    mov r1,a

    call TOGGLEXY
    ret

; writeaminutes - starting from a blank screen, we set it up
;     to display the current correct minute
; a - the minute to write
WRITEAMINUTES:
    mov r2,a
    jz WRITEAM2
WRITEAM:
    mov a,r2
    call DISPLAYAMINUTES
    djnz r2,WRITEAM
WRITEAM2:
    mov a,r2
    call DISPLAYAMINUTES
    ret

; displayaminutes - display analog minutes. We assume that the
;     screen has the current correct time on it
; a - the minute to display, fixed point, with one binary digit to
;   right of the binary point. 
; call this routine every 90 seconds, for nonjerky display of time.
;0 1 2 3 4 5 6 7 8 9 1011
;  # # # # # # # # # #   0
;#                     # 1 
;#                     # 2
;#                     # 3
;#                     # 4
;#         # #         # 5
;#         # #         # 6
;#                     # 7
;#                     # 8
;#                     # 9
;#                     # 10
;  # # # # # # # # # #   11
;0 1 2 3 4 5 6 7 8 9 1011
DISPLAYAMINUTES:
    mov r1,a			; first, convert to "LED" coordinates
    add a,r1			; double the accumulator
    clr c
    rrc a			; convert from 1 place fixed point to integer
    mov b,#3			; and divide by 3 (or multiply by .6666)
    div ab			; first time I ever heard about the b register

    cjne a,#5,$+3
    jc TOPRIGHT
    cjne a,#15,$+3
    jc RIGHT
    cjne a,#25,$+3
    jc BOTTOM
    cjne a,#35,$+3
    jc LEFT
    sjmp TOPLEFT
; perform the processing for the coordinates 
TOPLEFT:
    clr c
    subb a,#35			; make the led coordinate 0 based

    mov r0,#0			; get the y ...
    mov r1,#1			;    ...and x coordinates of the first led

    add a,r1			; modify them to reflect the desired led
    mov r1,a
    sjmp DISPMIN
LEFT:
    clr c
    subb a,#25

    mov r0,#10
    mov r1,#0

    xch a,r0
    subb a,r0
    mov r0,a
    sjmp DISPMIN
BOTTOM:
    clr c
    subb a,#15

    mov r0,#11
    mov r1,#10

    xch a,r1
    subb a,r1
    mov r1,a
    sjmp DISPMIN
RIGHT:
    clr c
    subb a,#5

    mov r0,#1
    mov r1,#11

    add a,r0
    mov r0,a
    sjmp DISPMIN
TOPRIGHT:
    mov r0,#0			; no normalization necessary
    mov r1,#6

    add a,r1
    mov r1,a
    sjmp DISPMIN
; add the absolute screen coordinates to the displacements that we have 
; just obtained
DISPMIN:
    inc r0		; just add 2
    inc r0
    inc r1		; just add 2
    inc r1
    call TOGGLEXY
    ret

; togglexy - toggle the bit at location (x,y) in the bitmap
; r0 contains the y coordinate, r1 contains the x coordinate
; (0,0) is in the upper left hand corner.(15,15) is the lower right.
; a r0 r1 dptr
TOGGLEXY:
    mov a,#BITMAP
    add a,r0		; compute an offset for the row
    add a,r0
    mov r0,a		; and get the byte

    mov a,r1		; compute an offset into the row
    cjne a,#8,$+3
    jc TOGGLEXY1	
    inc r0
TOGGLEXY1:
    mov dptr,#TOGGLETAB	; load a mask for the y coordinate
    mov a,r1
    movc a,@a+dptr
    xrl a,@r0		; and use it to toggle the bit
    mov @r0,a
    ret
; a variant of togglexy.
; don't toggle that bit. Set it.
SETXY:
    mov a,#BITMAP
    add a,r0		; compute an offset for the row
    add a,r0
    mov r0,a		; and get the byte

    mov a,r1		; compute an offset into the row
    cjne a,#8,$+3
    jc SETXY1	
    inc r0
SETXY1:
    mov dptr,#TOGGLETAB	; load a mask for the y coordinate
    mov a,r1
    movc a,@a+dptr
    orl a,@r0		; and use it to SET the bit
    mov @r0,a
    ret
; so shameful. 16 bytes, so that I may conveniently toggle bits
; without that painful shifting and counting
TOGGLETAB:
    db 10000000b
    db 01000000b
    db 00100000b
    db 00010000b
    db 00001000b
    db 00000100b
    db 00000010b
    db 00000001b
    db 10000000b
    db 01000000b
    db 00100000b
    db 00010000b
    db 00001000b
    db 00000100b
    db 00000010b 
    db 00000001b 
    
    db 00000001B
; alternate layout.
;# # # # # # # # # # # # # # # # 
;#             # #             # 
;#           #     #           #
;#         #         #         # 
;#       # # # # # # # #       # 
;#     # #             # #     # 
;#   #   #             #   #   # 
;# #     #             #     # # 
;# #     #             #     # # 
;#   #   #             #   #   # 
;#     # #             # #     # 
;#       # # # # # # # #       # 
;#         #         #         # 
;#           #     #           # 
;#             # #             # 
;# # # # # # # # # # # # # # # # 
