    $TITLE clockface

; Copyright (C) 1994 by Dan Liddell and Robert Armstrong.
; Copyright (C) 2004 by Spare Time Gizmos.
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful, but
; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
; ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
; Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, visit the website of the Free
; Software Foundation, Inc., www.gnu.org.

; improvements -
;   use the set bit commands, instead of the rotating right and left loops
;   document the depth of subroutine calls
; while I am mangling the timer variables, I should turn off the timer
;   interrupt
; problem with first LED (the 0th second) when setting with dip
;   switches (seems to be only during fast set mode)
; modularize clear screen code
; provide interface independant way to get at configuration switches, with
;     constants
; CHIME THE HOURS

; INITCLOCK - set the clock face to some appropriate starting place for
; xor writing operations, and set the intensity so power supply does not
; get too hot.
;
; clobbers ???
INITCLOCK:
    call CLEARSCREEN	; clear bitmap to zeros
    setb FLAG_DISPLAYCLOCK
    
    jb FLAG_TIMEISSET,INITCLOCK2

    mov DSECONDS,#0	; if time is not set, then it will not get past 0
    mov DMINUTES,#0
    mov	DHOURS,#0

    sjmp INITCLOCK3
INITCLOCK2:
    mov DSECONDS,SECONDS; update the display variables
    mov DMINUTES,MINUTES
    mov	DHOURS,HOURS
INITCLOCK3:
    jnb FLAG_TIMEISSET,INITCLOCK3A	; Karen says lonely second looks like
    					; a bug
    call WRITESECONDS	; opportunity for time display asynchrony
INITCLOCK3A:
    call DISPLAYDMINUTES; writeseconds looks at SECONDS, but we have
    call DISPLAYDHOURS	; taken readings from DSECONDS and DHOURS earlier
    
    ret

; this routine does the same stuff that initclock does, but if it is 
; called when the FLAG_TIMEISSET bit is false, then it zeros the times
; as well, so that they agree with the time that is displayed when the
; clock is not set (0:0:0). also sets the timeisset flag
NUKECLOCK:
    jb FLAG_TIMEISSET,NUKECLOCK2
    mov TICKS,#0
    mov SECONDS,#0
    mov MINUTES,#0
    mov HOURS,#0
    setb FLAG_TIMEISSET
NUKECLOCK2:
    jb FLAG_DIGITALCLOCK,NUKECLOCK3
    call AINITCLOCK
    ret
NUKECLOCK3:
    call INITCLOCK
    ret

; here we process user commands for setting the clock and the intensity
; of the leds and anything else that we can deal with. If not, then we
; just exit
SETCLOCK:
    cjne a,#CMD_BRIGHTNESS,SETCLOCKSTOP
    call LEDCHANGE
    ajmp SETCLOCKRET

SETCLOCKSTOP:		; should we just stop the clock?
    cjne a,#CMD_STOPCLOCK,SETCLOCKREGISTER
    mov TICKS,#0	; just wedge the timer, if so
    sjmp SETCLOCKRET

SETCLOCKREGISTER:
    cjne a,#CMD_ZEROSECONDS,SETCLOCK1AA
    call NUKECLOCK
    mov TICKS,#0	; to prevent unfortunate occurrence
    mov SECONDS,#0
    call ERASESECONDS
    sjmp SETCLOCKRET

SETCLOCK1AA:
    cjne a,#CMD_SETSECONDS,SETCLOCKSET1    
    call NUKECLOCK
    mov TICKS,#0	; to prevent unfortunate occurrences
    push HOURS
    push MINUTES
    lcall ONCE_A_SECOND	; to increment the time rapidly
    pop MINUTES
    pop HOURS
    sjmp SETCLOCKRET 

SETCLOCKSET1:
    cjne a,#CMD_SETMINUTES,SETCLOCKHOURS
    call NUKECLOCK
    mov TICKS,#0	; to prevent unfortunate occurrences
    push SECONDS
    push HOURS
    lcall ONCE_A_MINUTE	; to increment the time rapidly
    pop HOURS
    pop SECONDS
    sjmp SETCLOCKRET

SETCLOCKHOURS:
    cjne a,#CMD_SETHOURS,SETCLOCKJUSTEXIT
    call NUKECLOCK
    mov TICKS,#0	; to prevent unfortunate occurrences
    push SECONDS
    push MINUTES
    call ONCE_AN_HOUR
    pop MINUTES
    pop SECONDS
SETCLOCKRET:
    ret
SETCLOCKJUSTEXIT:	; return to callers caller
    pop ar0
    pop ar0
    ret

; CLOCKONCE - initialize the clock face, and display at least once. If
; it is allowed to stay running the clock, it polls the variables that
; keep track of time, and when they change, it changes the clock face.
; when it is called, it initializes the screen once, and then takes on
; incremental update duties.
; if it reads the next mode key, it returns it in a, so that it can be
; tested by the caller.FLAG_DISPLAYCLOCK is nonzero if need to update display
;
; clobbers (a,r0,r1,r2,r3,r4,r5,r6,r7), TICKS, SECONDS
; updates DSECONDS, DMINUTES, DHOURS from SECONDS, MINUTES, HOURS, 
;     FLAG_DISPLAYCLOCK
;
CLOCKONCE:
    setb FLAG_DIGITALCLOCK
    call INITCLOCK
    sjmp RUNCLOCK1


; RUNCLOCK1 - steady state clock running, assuming that we have
; already been initialized. 
;
RUNCLOCK1:
    call GETKEY		; get some potential user input
    jnz RUNCLOCK1A
    ajmp RUNCLOCKNOSET	; there is none
RUNCLOCK1A:
    call SETCLOCK

; here is the entry after we have done input processing, and now have to
; get to the clock display code
;
RUNCLOCKNOSET:
    jb FLAG_TIMEISSET,RCNS1	; refuse to do anything else, until the
    sjmp RUNCLOCKEXIT		; time is actually set

RCNS1:
    mov a,SECONDS
    clr c
    subb a,DSECONDS	; have the seconds changed since last display?
    jz RUNCLOCK2	; no

    mov DSECONDS,SECONDS; yes. update the bitmap and display it
    mov r2,DSECONDS
    mov r3,#0
    call DISPLAYSECONDS
    setb FLAG_DISPLAYCLOCK	; need to do an update
RUNCLOCK2:
    mov a,MINUTES
    clr c
    subb a,DMINUTES	; have the minutes changed since last display?
    jz RUNCLOCK3	; no

    call DISPLAYDMINUTES; yes. update the bitmap
    mov DMINUTES,MINUTES
    call DISPLAYDMINUTES
    setb FLAG_DISPLAYCLOCK	; need to display clock

RUNCLOCK3:
    mov a,HOURS
    clr c
    subb a,DHOURS	; have the hours changed since last display?
    jz RUNCLOCKEXIT

    call DISPLAYDHOURS	; yes. update the bitmap
    mov DHOURS,HOURS
    CALL DISPLAYDHOURS
    setb FLAG_DISPLAYCLOCK	; need to display the clock

RUNCLOCKEXIT:
    clr a		; say that no key event made us leave.
RUNCLOCKJUSTEXIT:	; the user typed something we don't understandae.
    ret			; let the caller deal with it


; CLOCKUNTIL - run the clock until asked to leave. update the display if
; RUNCLOCK1 told us it was necessary.
; FLAG_DISPLAYCLOCK used on entry to determine if fadeup is done.
CLOCKUNTIL:
    setb FLAG_DIGITALCLOCK
    call INITCLOCK
    call DISPLAY_UPDATE
    jnb FLAG_DOFADEUP,CLOCKU	; do fadeup?
    call FADEUP			; yes.
    jz CLOCKU
    ret
CLOCKU:
    clr FLAG_DISPLAYCLOCK; we want to know if we have to update display
    call RUNCLOCK1
    jz CLOCKDISPLAY
    ret			; had to leave. command button pressed
CLOCKDISPLAY:
    jnb FLAG_DISPLAYCLOCK,CLOCKU
    call DISPLAY_UPDATE	; had to update
    sjmp CLOCKU


; DISPLAYDMINUTES - display or erase (using xor mode) the tens and ones
; digits for the minutes display
; 
; clobbers a,r0,r1,r2,r3,r4,r5,r6,DPTR,carry
DISPLAYDMINUTES:
    mov a,DMINUTES
    call DIV10
    mov a,r1
    mov r2,a
    mov r1,#BITMAP+18   ; erase/write the tens digit
    mov r3,#1
    mov r4,#0
    call DISPLAYDIGIT

    mov a,DMINUTES
    call DIV10
    mov r2,a
    mov r1,#BITMAP+19   ; erase/write the ones digit
    mov r3,#0
    mov r4,#1
    call DISPLAYDIGIT
    ret

; DISPLAYDHOURS - displays or erases the tens and ones digits for the 
; hours display. Implements the behaviors:
;   double zero hours between midnight and 1 am
;   centered single digit hour display between 1 and 10 am
;   conventional double digit display after 9 am
; this code is somewhat complicated.
;
; clobbers ???
DISPLAYDHOURS:
    mov a,DHOURS	; if it is between midnight and 1 am, 
    jz DISPLAYDHOURS1	; then do not suppress the leading zero

    call DIV10
    cjne r1,#0,DISPLAYDHOURS1; branch if we are displaying double hour digits

    mov r7,a		; otherwise, we will display a single centered digit

    mov r2,a		; erase/display left half of the single hour digit
    mov r1,#BITMAP+4
    mov r3,#4
    mov r4,#0
    call DISPLAYDIGIT

    mov a,r7

    mov r2,a		; erase/display right half of the single hour digit
    mov r1,#BITMAP+5
    mov r3,#0
    mov r4,#4
    call DISPLAYDIGIT
    ret

DISPLAYDHOURS1:		; display two digits for the hour
    mov a,DHOURS
    call DIV10
    mov a,r1
    mov r2,a
    mov r1,#BITMAP+4    ; erase/display the old tens digit
    mov r3,#1
    mov r4,#0
    call DISPLAYDIGIT

    mov a,DHOURS
    call DIV10
    mov r2,a
    mov r1,#BITMAP+5    ; erase/display the old ones digit
    mov r3,#0
    mov r4,#1
    call DISPLAYDIGIT
    ret

; ERASESECONDS - write zeros to the LEDs on the perimeter that are used
; to display the seconds. Nothing else is disturbed on the screen
;
; clobbers  (a, r0, r1,) r2, r3, DSECONDS
;
ERASESECONDS:
    mov r2,#59
ERASESECONDSLOOP:
    mov DSECONDS,r2
    mov r3,#1
    push ar2
    call DISPLAYSECONDS
    pop ar2
    djnz r2,ERASESECONDSLOOP
    call DISPLAYSECONDS
    ret

; WRITESECONDS - given the current second count in DSECONDS, turns on
; all but one of the perimeter LEDS required to represent that time.
; 
; clobbers (a, r0, r1,) r2, r3, 
;
WRITESECONDS:
    mov r2,#0
WRITESECONDSLOOP:
    mov r3,#0
    push ar2
    call DISPLAYSECONDS
    pop ar2
    mov a,r2
    cjne a,DSECONDS,WRITESECONDSLOOP1
    ret
WRITESECONDSLOOP1:
    inc r2
    sjmp WRITESECONDSLOOP


; DIV10 - take the value in the accumulator, and return mod and div 10
;
; arguments are 
; a - the number to break up. 
;
; returns
; a - mod 10 
; r1 - div 10
DIV10:
    mov r1,#0		; initialize the count of tens...
DIV101:
    cjne a,#10,DIV102   ; if not equal to 10...
    inc r1		; well. equal to 1. increment the count, return
    clr a
DIV103:
    ret
DIV102:
    jc DIV103	 	; then if it was less than 10...
    clr c
    subb a,#10		; well. greater than 10.  
    inc r1
    sjmp DIV101


; DISPLAYDIGIT - shamelessly stolen from load_bitmap, this routine
; takes a bitmap and xors it onto every other byte of the destination.
; good for creating or erasing digits on the clock face
;
; r1 contains the address of the first byte in ram to be mangled. Therafter,
;   every other byte is affected
; r2 contains the digit that should be displayed in the ram
; r3 is the number of bits to shift the byte to the right before xoring
; r4 is the number of bits to shift the byte to the left before xoring
;  r3 and r4 zero means no shifting
; clobbers a,dptr,r5,r6
;
DISPLAYDIGIT:
    mov a,r3		; save shift count
    mov r5,a
    mov a,r4		; save shift count
    mov r6,a
    MOV R0,#5	  	; set up count to always xor 5
    mov a,r2		; compute offset for digit table (multiply by 5)
    add a,r2
    add a,r2
    add a,r2
    add a,r2
    mov r2,a		
    mov	dptr,#CLOCKDIGITS
DISPLAYDIGIT1:
    mov a,r5		; get shift count
    mov r3,a
    mov a,r6		; get shift count
    mov r4,a

    mov A,r2		; we have to have an offset
    MOVC A,@A+DPTR	;  to access program memory
    
    cjne r3,#0,DISPLAYDIGIT4
    cjne r4,#0,DISPLAYDIGIT2
    sjmp DISPLAYDIGIT3	; no shifting
DISPLAYDIGIT4:		; shifting left
    clr c	
    rrc a
    djnz r3,DISPLAYDIGIT4
    sjmp DISPLAYDIGIT3
DISPLAYDIGIT2:
    clr c		
    rlc a
    djnz r4,DISPLAYDIGIT2
DISPLAYDIGIT3:
    xrl a,@r1		; xor the row onto what we have
    MOV @R1,A		; copy it to the bitmap
    INC R1		; on to the byte after next byte
    INC R1
    INC DPTR		; get the next byte from the bitmap
    DJNZ R0,DISPLAYDIGIT1
    RET 


; DISPLAYSECONDS is a routine that will cause the corner pixels to light
; as the seconds slip past. it just xors the bits, so that the next time
; around, the seconds will go out one at a time
; if R3 is not zero, then invert the mask that we construct, and use it
; as an and mask instead, to clear the bit that we want to.
;
; takes r2 as the second to write
; clobbers a,r0,r1,r3
;
DISPLAYSECONDS:
    mov a,r2

    add a,#8		; shift everything over 8 to the right, so that zero
    cjne a,#60,$+3	; is in the top of the screen
    jc DS2
    subb a,#60
DS2:
    mov r2,a

    cjne a,#46,$+3	; is the second in the left edge?
    jc RCD1
    mov a,#60		; construct a row index. range is 1-14
    subb a,r2
    rl a		; multiply by two to get a byte offset
    add a,#BITMAP	; get the address of the byte to change
    mov r0,a		;   into a register that we can use for indirect
    mov a,@r0		; get the byte to alter
    cjne r3,#0,DSAND	; check to see if and or xor
    xrl a,#10000000B
    mov @r0,a
    ret
DSAND:
    anl a,#01111111B
    mov @r0,a
    ret

RCD1:
    cjne a,#38,$+3	; is the second in the lower left
    jc RCD2
    mov a,#45		; begin construction of a bit index
    subb a,r2
    mov r1,a
    inc r1
    mov r0,#BITMAP+30	; get the address to change
    setb c		; set the carry bit, to construct the xor mask
    clr a
RCD1A:
    rrc a
    djnz r1,RCD1A
    cjne r3,#0,RCD1AAND	; check to see if we should clear it or xor it
    xrl a,@r0
    mov @r0,a
    ret
RCD1AAND:
    cpl a
    anl a,@r0
    mov @r0,a
    ret

RCD2:
    cjne a,#30,$+3	; is the second in the lower right?
    jc RCD3
    mov a,#37		; begin construction of a bit index
    subb a,r2
    inc a
    mov r1,a
    mov r0,#BITMAP+31	; get the address to change
    setb c		; set the carry bit, to construct the xor mask
    clr a
RCD2A:
    rrc a
    djnz r1,RCD2A
    cjne r3,#0,RCD2AAND
    xrl a,@r0
    mov @r0,a
    ret
RCD2AAND:
    cpl a
    anl a,@r0
    mov @r0,a
    ret

RCD3:
    cjne a,#16,$+3	; is the second in the right edge?
    jc RCD4
    mov a,r2	; begin construction of a row index
    subb a,#15
    rl a		; double it, to make it a byte offset and
    inc a		;   then add 1 to get address of second byte in row
    add a,#BITMAP	; get the address of the byte to alter
    mov r0,a
    mov a,@r0		; get the byte to alter
    cjne r3,#0,RCD3AND	; decide to and or xor it
    xrl a,#00000001B
    mov @r0,a
    ret
RCD3AND:
    anl a,#11111110B
    mov @r0,a
    ret

RCD4:
    cjne a,#8,$+3	; is the second in the upper right edge?
    jc RCD5
    mov a,r2	; begin construction of a bit index
    subb a,#8
    inc a
    mov r1,a
    mov r0,#BITMAP+1	; get the address to change
    setb c		; set the carry bit, to construct the xor mask
    clr a
RCD4A:
    rrc a
    djnz r1,RCD4A
    cjne r3,#0,RCD4AAND	; decide whether or xor or and
    xrl a,@r0
    mov @r0,a
    ret
RCD4AAND:
    cpl a
    anl a,@r0
    mov @r0,a
    ret

RCD5:
    mov r1,ar2	; the second must be in the upper left edge
    inc r1
    setb c		; set the carry bit, to construct the xor mask
    clr a
RCD5A:
    rrc a		; construct the mask
    djnz r1,RCD5A

    mov r0,#BITMAP
    cjne r3,#0,RCD5AAND	; decide whether to and or xor

    xrl a,@r0
    mov @r0,a
    ret
RCD5AAND:
    cpl a
    anl a,@r0
    mov @r0,a
    ret


; CLOCKDIGITS - the font for the clock face. each entry is 5 bytes
; long, and the leftmost and rightmost bits of each line must be 
; 0 (unless you want to damage the seconds display while you are
; updating the clock).
; this font is pretty ugly, but it fits within the space required
CLOCKDIGITS:
    db	00111100B
    db	00100100B
    db	00100100B
    db	00100100B
    db	00111100B

    db	00011000B
    db	00011000B
    db	00011000B
    db	00011000B
    db	00011000B

    db	00111100B
    db	00000100B
    db	00111100B
    db	00100000B
    db	00111100B

    db	00111100B
    db	00000100B
    db	00011100B
    db	00000100B
    db	00111100B

    db	00100100B
    db	00100100B
    db	00111100B
    db	00000100B
    db	00000100B

    db	00111100B
    db	00100000B
    db	00111100B
    db	00000100B
    db	00111100B

    db	00111100B
    db	00100000B
    db	00111100B
    db	00100100B
    db	00111100B

    db	00111100B
    db	00000100B
    db	00000100B
    db	00000100B
    db	00000100B

    db	00111100B
    db	00100100B
    db	00111100B
    db	00100100B
    db	00111100B

    db	00111100B
    db	00100100B
    db	00111100B
    db	00000100B
    db	00000100B


PLEASEWAIT:
    db 00000000B,00000000B
    db 00000000B,00000000B
    db 00100010B,00111000B
    db 00100010B,01000100B
    db 00101010B,01111100B
    db 00110110B,01000100B
    db 00100010B,01000100B
    db 00000000B,00000000B
    db 00000000B,00000000B
    db 00000000B,00000000B
    db 00011100B,01111100B
    db 00011100B,00010000B
    db 00011100B,00010000B
    db 00011100B,00010000B
    db 00011100B,00010000B
    db 00000000B,00000000B


; very clever life clock, which starts from random crud, and then generates
; a time from it. instead of a hairy algorithm, set seed the bitmap with
; the pattern, run it 30 or so generations into the future, and display that.
; then, we seed the bitmap with the same thing, and then only run it 29
; generations into the future. and so on. bunch of logistical problems, like
; what happens if the time changes before we are done, what happens if
; it dies in only 3, generations, and so on. Bob's idea.
;
; well, what Bob really wanted was to have life run forward, and coalesce
; into the clock as the last generation. pretty hard to compute. I have
; just run life backwards, to achieve the same chaos-into-order thing
CLOCKLIFE:
    setb FLAG_DOFADEDOWN		; say there is no need to fadedown
    					;  when we perform the next display 
					;  display update (because we have
					;  already emptied the screen)
    MOV DPTR,#PLEASEWAIT
    call load_bitmap
    call DISPLAY_UPDATE			;  compute the first image. don't want
    call fadeup				; lose user input

    					;  stuff from previous mode hanging 
					;  around while we are doing it
    clr FLAG_DISPLAYLIFE		; turn off life display
    mov SPEED,#1			; almost as fast as we can. 
    					; we must remain responsicve

CLOCKLIFE1:
    call CLOCKONCE			; set up display variables.
    jz CLOCKLIFE1A
    call LCCOM				; handle user input
CLOCKLIFE1A:
    call clearscreen			; now, count up until we find out
    call DISPLAYDMINUTES		; where this dies, or until we hit
    call DISPLAYDHOURS			; our generation limit
    mov LCINDEX,#0			
CLFIRST:
    mov a,LCINDEX
    cjne a,#30,CLSECOND			; are we at 30 generations yet?
    sjmp CLOCKLIFELOOP			; yes. get to work.
CLSECOND:
    inc LCINDEX
    call nextgen
    call getkey				; check for user input
    jz CLSECONDA
    call LCCOM				; handle user input
CLSECONDA:    
    mov a,POP0				; get the population
    jnz CLFIRST				; if not zero, 
; now we have a count. let's start doing it backwards.
CLOCKLIFELOOP:
    call CLEARSCREEN			; clear the screen
    call DISPLAYDMINUTES		; write the time into it, without disp
    call DISPLAYDHOURS
    mov a,LCINDEX			; check to see if we are done warping
    jnz CLOCKLIFENEXT			;  and need to just display the time.

    call DISPLAY_UPDATE			; ... and just display the time as it
    setb FLAG_DOFADEDOWN		; leave display up until we have new 
    setb FLAG_DOFADEUP

    mov a,#HERTZ/2
    call WAIT
    jz CLOCKLIFE1
    call LCCOM
    sjmp CLOCKLIFE1			; get the next time
CLOCKLIFENEXT:
    mov MIN0,LCINDEX
CLOCKLIFENEXT1:
    call RUNJUST
    jz CLOCKLIFECONT			; check for user input
    call LCCOM				; handle user input
    sjmp CLOCKLIFENEXT1			; and continue computation if needed
CLOCKLIFECONT:
    jnb FLAG_DOFADEDOWN,CLOCKLIFECONT1
    call fadedown
    clr FLAG_DOFADEDOWN
    jz CLOCKLIFECONT1			; check for user input
    call LCCOM				; handle user input
CLOCKLIFECONT1:
    call DISPLAY_UPDATE			; display the results
    jnb FLAG_DOFADEUP,CLOCKLIFEDEC	; force a fadeup after update?
    call fadeup				; yes.
    clr FLAG_DOFADEUP			; 
    jz CLOCKLIFEDEC			; check for user input
    call LCCOM				; handle user input
CLOCKLIFEDEC:
    dec LCINDEX

    mov a,#30				; insert enough delay here so the
    clr c				; user can see what is happening.
    subb a,LCINDEX
;    rl a				; multiply by two...
    rl a				; multiply by two...
    call wait				; should compute a function here
    					; based on HERTZ, but this is ok
    jz CLOCKLIFELOOP
    call LCCOM
    sjmp CLOCKLIFELOOP

; helper routine for lifeclock. wait, and then run one generation.
; can read input
ONEFACE:
    call wait
    jz ONEFACE1		; see if "next command" pressed
    ret
ONEFACE1:
    mov SPEED,#1
    mov MIN0,#1
    call RUNJUST
    ret
;
; the infamous life clock, warping and dripping. nice, smooth transition
LIFECLOCK:
    setb FLAG_DISPLAYLIFE		; say we want generations displayed
    call CLOCKONCE			; force a run of the clock,and display
    jz LIFECLOCKA			; was there a command to process?
    call LCCOM
LIFECLOCKA:
    call ERASESECONDS			; reduce it to just hours and minutes
    mov BITMAP,#0			; erase that last little pixel,
    call DISPLAY_UPDATE			; and update the screen.

    jnb FLAG_DOFADEUP,LCB		; force a fadeup on entry?
    call fadeup				; fade up from black so user can see.
    jz LCB				; handle user input
    call LCCOM
LCB:					; wait for a while longer
    mov a,#HERTZ/2
    call WAIT
    jz LCB1
    call LCCOM
LCB1:
    mov SPEED,#HERTZ/2

    mov a,#HERTZ
    call WAIT
    jz LCLOOP				; check for user input
    call LCCOM
LCLOOP:
    mov min0,#1
    call RUNJUST
    jz LCLOOP1				; any input?
    call LCCOM
LCLOOP1:
    mov a,SPEED
    clr c
    subb a,#HERTZ/60			; better divide hertz/2 evenly
    jz LC7				; go to fast mode
    mov SPEED,a
    mov a,POP0				; check population
    jnz LCLOOP				; exit if 
    sjmp LCNOTFIN
LC7:
    mov	SPEED,#HERTZ/60
    mov GEN1,#00000010B		; maximum number of generations
    mov GEN0,#00000000B		; plus some more
LC7A:
    call RUNUNTIL
    jz LCNOTFIN
    call LCCOM			; handle user input
    sjmp LC7A			; and continue running, if necessary
LCNOTFIN:
    mov a,GEN0
    jnz LCNOTFIN4
    mov a,GEN1
    jnz LCNOTFIN4
    call SSSHORT_BEEP	; alert the implementor of a timeout (bug)
LCNOTFIN4:
    mov a,#HERTZ
    call wait
    jz LCNOTFIN3
    call LCCOM
LCNOTFIN3:
    call fadedown
    jz LCNOTFIN2
    call LCCOM
LCNOTFIN2:
    setb FLAG_DOFADEUP
    ajmp LIFECLOCK

; handle commands to life game here. If we can successfully 
; deal with the command here, then we will just return. If we 
; cannot, then we will pop our return address off of the stack,
; and then return to that, which will be the caller
LCCOM:
    cjne a,#CMD_BRIGHTNESS,LCCOM1
    call LEDCHANGE
    ret
LCCOM1:
    pop ar0		; just some unfortunate register to get rid of the
    pop ar0		; current return address...
    ret			; now return to our caller's caller

