	$TITLE	(Simple Debug Routines)


; Copyright (C) 1994 by Dan Liddell and Robert Armstrong.
; Copyright (C) 2004 by Spare Time Gizmos.
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful, but
; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
; ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
; Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, visit the website of the Free
; Software Foundation, Inc., www.gnu.org.

;   This module contains some simple debugging routines using the serial
; port on the 8051.  The DEBUG_STARTUP routine initializes the serial port 
; hardware and sets up Timer 1 as the baud rate generator (this assumes,
; naturally, that the code isn't using Timer 1 for anything else!).  The
; remaining routines output various ASCII and hex values to the serial port.

; DEBUG_STARTUP
;   Initialize the serial port hardware and set up Timer 1 as the baud
; rate generator.  The actual baud rate used is determined by the BAUD
; constant in SYMBOLS.ASM.  Note that serial port interrupts are NOT
; used...
;
DEBUG_STARTUP:
	MOV	SCON, #52H	; serial port mode 1, set TI
	ANL	TMOD, #0FH	; clear timer 1
	ORL	TMOD, #20H	; and select timer 1 model 2 (auto reload)
	MOV	TH1, #BAUD	; set the baud rate
	ANL	PCON, #7FH	; be sure that SMOD is cleared
	SETB	TR1		; and start the timer
	SETB	C		; always return success
	RET			; and we're done


; TCHAR
;   Type one character on the console terminal.  Note that this routine
; BLOCKS if the serial port is currently busy, so don't use it in any
; timing sensitive locations!
;
TCHAR:
	JNB	TI, TCHAR	; wait for the TI bit to set
	CLR	TI		; then clear it for next time
	MOV	SBUF, A		; type this character
	RET			; and we're done

; TCRLF
;   Type a CRLF on the console terminal...
;
TCRLF:
	MOV	A, #0DH		; carriage return
	CALL	TCHAR		; ...
	MOV	A, #0AH		; line feed
	JMP	TCHAR		; type that and return

; TSTRING
;   Type an ASCIZ string (the address of which is in DPTR)...
;
TSTRING:
	CLR	A		; zero the offset
	MOVC	A, @A+DPTR	; fetch the next character from the string
	INC	DPTR		; and advance the pointer
	JZ	TSTR99		; quit when we find a zero
	CALL	TCHAR		; it's a real character - type it
	JMP	TSTRING		; and keep going
TSTR99:	RET			; here when we find the end of the string

; THEX1
;   Convert the ACC into a single hex digit...
;
THEX1:	ANL	A, #0FH		; insure the value is only 4 bits
	CJNE	A, #0AH, $+3	; jump if ACC .LT. 10
	JC	THEX11		; ....
	ADD	A, #'A'-'9'-1	; ACC .GE. 10 - add offset for letters
THEX11:	ADD	A, #'0'		; convert binary to ASCII
	JMP	TCHAR		; and type it...

; THEX2
;   Convert the ACC into a two digit hex value...
;
THEX2:	PUSH	ACC		; save the AC for a minute
	SWAP	A		; print the high nibble first
	CALL	THEX1		; ...
	POP	ACC		; then print the low nibble
	JMP	THEX1		; ...
