	$TITLE	(The Game of Life)

; Copyright (C) 1994 by Dan Liddell and Robert Armstrong.
; Copyright (C) 2004 by Spare Time Gizmos.
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful, but
; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
; ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
; Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, visit the website of the Free
; Software Foundation, Inc., www.gnu.org.

; lightweight development directives
	$PRINT

;   Life ??  Don't talk to me about life.  I've seen it.  It's rubbish...
;						- Marvin
;
;   Marvin has just seen life. He didn't have life. It makes a difference.
;						- Dan L
;
	$MOD51
$EJECT
	$INCLUDE(SYMBOLS.ASM)
$EJECT
	$INCLUDE(RAM.ASM)
$EJECT
	CSEG AT 0000H

;   These are the startup and interrupt vectors.  They MUST reside in low
; ROM at the locations specified!

; System startup (reset) vector...
	ORG	00H
	LJMP	SYSTEM_STARTUP

; Infrared receiver interrupt (external interrupt 0)...
	ORG	03H
	LJMP	IR_INTERRUPT

; Timer 0 interrupt (used for infrared RC5 interpretation)...
	ORG	0BH
	LJMP	TIMER0_INTERRUPT

; Line time clock interrupt (external interrupt 1)...
	ORG	13H
	LJMP	CLOCK_INTERRUPT

; Timer 1 interrupt...
	ORG	1BH
;	LJMP	TIMER1_INTERRUPT

; Serial port interrupt...
	ORG	23H
;	LJMP	SERIAL_INTERRUPT

; End of the interrupt vectors...
	ORG	30H

$EJECT
	$TITLE	(System Startup)

;   This code is entered after a hardware reset or power on.  We need
; to initialize all the hardware and software and then start playing
; the game of life!
SYSTEM_STARTUP:

;   NOTE: a hardware reset on the 8051 gives us:
;
;	PC = A = B = PSW = IE = IP = SCON = SBUF = DPTR = 0
;	SP = 7
;
; Everything else is undefined...

; Should we have a POST for the basic CPU itself ???
;???	CLR	CMOD		; turn OFF the WDT!
;???	AUXR = EXTRAM | A0;

;   Clear all RAM locations to zero.  This ensures that any uninitialized
; variable bugs don't cause unrepeatable behavior - the same thing will
; always happen every time!
	MOV	R0, #1		; Remember that R0 is RAM location 0!
SYSIN1:	MOV	@R0, #0		; clear another byte
	INC	R0		; on to the next
	CJNE	R0,#080H,SYSIN1	; do them all

;   Initialize the stack pointer...   Note that since a PUSH operation
; pre-inccrements the pointer, we want to use STACK-1, not STACK to avoid
; wasting a byte at the top of the stack.  Every byte counts!!
	MOV	SP, #STACK-1	; ...

;   Enable the interrupt system, but disable all the individual sources.
; These get turned on one at a time as we initialize the associated
; software...
	MOV	IE, #80H	; ...

; Turn off the damn buzzer !!!!
	BUZZER_OFF

; Initialize the debugging console...
	CALL	DEBUG_STARTUP	; no error return here
	MOV	DPTR, #HLOMSG	; type our copyright information...
	CALL	TSTRING		; ...

;   Start up the line time clock.  Some of the other startup routines use
; the clock, so this must always be first!
	INC	POST_CODE	; POST CODE #1
	CALL	CLOCK_STARTUP	; start the clock
	JNC	POST_FAILED	; jump if the clock isn't working

; Start up the display...
	INC	POST_CODE	; POST CODE #2
SYSIN2:	CALL	DISPLAY_STARTUP	; ...
	JNC	POST_FAILED	; jump if the display isn't working

; Initialize the RC5 receiver...
	INC     POST_CODE	; post code #3
	CALL    IR_STARTUP	; turn on the RC5 receiver
	JNC	POST_FAILED	; jump if the receiver isn't working

; Play a little tune to signal successful completion!!!
	CALL	LONG_BEEP

;   All startup functions are completed and we're ready to start the
; background processing.  DAN, this is where you should put a long jump
; to your code...
	LJMP	MAIN		; and away we go!!


;   We get here if one of the POST functions fails.  We use the beeper
; to signal our unhappiness.  Note that the beeper doesn't have its
; own explicit POST - we'll just have to assume that it works.  The
; idea is to sound N sort tones followed by a longer pause, and then
; repeat.  The number of short tones is determined by the POST code
; that failed (e.g. 1 = clock, 2 = display, etc).  Since we can't
; use the line time clock here, we generate the delays with programmed
; loops.  This will vary with the CPU clock speed, but it'll have to
; do.
POST_FAILED:
	MOV	R2, POST_CODE	; get the code we want to signal
POSTF1:	BUZZER_ON		; then turn on the beeper
	MOV	R1, #119	; this loop requires about 100 milliseconds
POSTF2:	MOV	R0, #0		; ...
	DJNZ	R0, $		; ...
	DJNZ	R1, POSTF2	; ...
	BUZZER_OFF		; then let's have the same amount of silence
	MOV	R1, #119	; ...
POSTF3:	MOV	R0, #0		; ...
	DJNZ	R0, $		; ...
	DJNZ	R1, POSTF3	; ...
	DJNZ	R2, POSTF1	; make the correct number of beeps
	MOV	R1, #0		; a long delay now - almost 1/4 of a second
POSTF4:	MOV	R0, #0		; ...
	DJNZ	R0, $		; ...
	DJNZ	R1, POSTF4	; ...
	SJMP	POST_FAILED	; then do it all over again
$EJECT
	$INCLUDE(DEBUG.ASM)
	$INCLUDE(UTILS.ASM)
	$INCLUDE(DISPLAY.ASM)
	$INCLUDE(CLOCKFAC.ASM)
	$INCLUDE(NEXTGEN.ASM)
	$INCLUDE(CLOCK.ASM)
	$INCLUDE(INFRARED.ASM)
	$INCLUDE(ANALOG.ASM)
	$INCLUDE(RANDOM.ASM)

; and the fun begins...
MAIN:
	mov     a,#15
	call    SET_INTENSITY
	call COPYRIGHT		; always display once, debug
	CALL	FADEDOWN
	setb FLAG_DOFADEUP
	clr FLAG_TIMEISSET	; say that the time has not been set yet
MAIN1:
	call LIFEDEMO		; display interesting patterns
	call ALLMODES
	jnz MAIN2
	sjmp MAIN1
MAIN2:	
	call CLOCKUNTIL		; just display the clock
	call ALLMODES
	jnz MAIN6
	sjmp MAIN2
MAIN6:
	call ACLOCKUNTIL	; the analog clock
	call ALLMODES
	jnz MAIN3
	sjmp MAIN6
MAIN3:
	call LIFECLOCK		; run life forward from time
	call ALLMODES
	jnz MAIN4
	sjmp MAIN3
MAIN4:
	call CLOCKLIFE		; run life backward, to from time
	call ALLMODES
	jnz MAIN5
	sjmp MAIN4
MAIN5:
	call DAZZLE		; just random on the screen
	call ALLMODES
	jnz MAIN8
	sjmp MAIN5
MAIN8:
        call LIFEBURN		; remote control random life
	call ALLMODES
	jnz MAIN7
	sjmp MAIN8
MAIN7:
	jnz MAIN1
	sjmp MAIN7

; handle commands that work in all modes, and the next mode command,
; at the top level


ALLMODES:
    cjne a,#CMD_COPYRIGHT,ALLMODES2
    call copyright
    setb FLAG_DOFADEUP
    sjmp ALLMODESEXIT

ALLMODES2:
    setb FLAG_DOFADEUP
    cjne a,#CMD_SHOWMENU,ALLMODES3
    call MENUDISPLAY
    sjmp ALLMODESEXIT
; mode change. do the normal stuff
ALLMODES3:    
    setb FLAG_DOFADEUP
    cjne a,#CMD_NEXTMODE,ALLMODES4
    call segue2		; toss character. not good.
    call fadedown	; toss this one too. not good.
    sjmp ALLMODESJUSTEXIT
ALLMODES4:
    clr FLAG_DOFADEUP	; prevent a fadeup
    call LONG_BEEP
ALLMODESEXIT:
    clr a
    ret
ALLMODESJUSTEXIT:
    mov a,#CMD_NEXTMODE		; anything nonzero should be ok
    ret

MENUDISPLAY: ret

; alter the LED intensity. Bad, using an illegal value in the intensity
; register, to indicate display off. 16 means display off.
LEDCHANGE:
    dec INTENSITY
    mov a,INTENSITY
    cjne a,#255,LEDCHANGE1	; were we at zero?
    mov INTENSITY,#16		; next decrement will get us to a legal val
    call DISPLAY_OFF
    ret
LEDCHANGE1:			; no, we were not at zero.
    cjne a,#15,LEDCHANGE2	; did we decrement from "off" to on?
    call SET_INTENSITY
    call DISPLAY_ON
    ret
LEDCHANGE2:
    mov a,INTENSITY
    cjne a,#14,LEDCHANGE2A
    mov a,#12
    sjmp LEDCHANGE3
LEDCHANGE2A:
    cjne a,#11,LEDCHANGE2B
    mov a,#9
    sjmp LEDCHANGE3
LEDCHANGE2B:
    cjne a,#8,LEDCHANGE2C
    mov a,#6
    sjmp LEDCHANGE3
LEDCHANGE2C:
    cjne a,#5,LEDCHANGE3
    mov a,#3
    sjmp LEDCHANGE3
LEDCHANGE3:
    call SET_INTENSITY
    ret
    
; I am kind of partial to gasification (spread of whatever on the screen
; now into its component atoms. A nice function of motion for each
; particle, like the lionheart dissolve, would be nice). 
; a "compress" would be nice, too. The active pixels on the screen would
; be compressed into a small bundle in the middle, which would then fade
; out. Kind of reminiscent of an old crt on a television set.

; the next generation. this collapses the screen from top and bottom, 
; sweeping the pixels along with it! I think Karen suggested this one.
;
SEGUE2:
    mov r0,#BITMAP		; point to the beginning of two rows,
    mov r1,#BITMAP+31		; the top and the bottom
SEGUE2LOOP:
    call SEGUE2ORTOP		; compress a line from top to bottom
    call SEGUE2ORTOP

    call SEGUE2ORBOT		; compress a line from bottom to top
    call SEGUE2ORBOT

    push ar0			; save these registers while we do the
    push ar1			; horizontal compressions
    call DISPLAY_UPDATE
    mov a,#HERTZ/60
    call wait
    pop ar1
    pop ar0
    jnz SEGUE2RET		; if token entered on remote control, exit

    cjne r0,#BITMAP+14,SEGUE2LOOP
    clr a
SEGUE2RET:
    ret
; segue2 helper routines
;
SEGUE2ORTOP:			; or a byte in with the one beneath it
    mov a,@r0			; get the byte
    mov @r0,#0			; and then clear it
    inc r0
    inc r0
    orl a,@r0			; or it with the one beneath it
    mov @r0,a			; and put it there
    dec r0			; point to the next byte
    ret
SEGUE2ORBOT:			; same as segue2orr0, but with the byte above
    mov a,@r1
    mov @r1,#0
    dec r1
    dec r1
    orl a,@r1
    mov @r1,a
    inc r1
    ret

; just write out the legal stuff, properly perverted
; 
COPYRIGHT:
	setb FLAG_DISPLAYLIFE	; say that we wish to display the generations

	push INTENSITY		; save whatever they had

	mov     a,#15		; and crank it up bright
	call    SET_INTENSITY

	MOV	DPTR, #COPYRIGHT_A
	CALL	LOAD_BITMAP
	CALL	DISPLAY_UPDATE
	call	FADEUP

	mov a,#HERTZ
	call HARDWAIT

	mov 	SPEED,#0
	mov GEN0,#0
	mov GEN1,#15
	call	RUNUNTIL
	call	FADEDOWN

	MOV	DPTR, #COPYRIGHT_B
	CALL	LOAD_BITMAP
	CALL	DISPLAY_UPDATE
	call	FADEUP

	mov a,#HERTZ/2
	call HARDWAIT

	MOV	DPTR, #COPYRIGHT_BA
	CALL	LOAD_BITMAP
	CALL	DISPLAY_UPDATE

	mov a,#HERTZ/2
	call HARDWAIT

	mov 	SPEED,#0
	mov GEN0,#0
	mov GEN1,#15
	call	RUNUNTIL
	call	fadedown

	MOV	DPTR, #COPYRIGHT_C
	CALL	LOAD_BITMAP
	CALL	DISPLAY_UPDATE
	call	FADEUP

	mov a,#HERTZ/2
	call HARDWAIT

	MOV	DPTR, #COPYRIGHT_CA
	CALL	LOAD_BITMAP
	CALL	DISPLAY_UPDATE

	mov a,#HERTZ/2
	call HARDWAIT

	mov 	SPEED,#0
	mov GEN0,#0
	mov GEN1,#15
	call	RUNUNTIL
	call	FADEDOWN

	MOV	DPTR,#PTL0
	CALL	LOAD_BITMAP
	CALL	DISPLAY_UPDATE
	call	FADEUP

	mov a,#HERTZ
	call HARDWAIT

	mov 	SPEED,#0
	mov GEN0,#0
	mov GEN1,#15
	call	RUNUNTIL
	call	FADEDOWN

	call CLEARSCREEN	; clear off remnants, for nice transition
	call DISPLAY_UPDATE

	pop acc			; set the intensity to what they had before
	call SET_INTENSITY
	ret

COPYRIGHT_A:
	db	00000000B,00000000B,
	db	00000111B,11100000B,
        db	00010000B,00001000B,
	db	00100011B,11000100B,
	db	00000100B,00100000B,
	db	01001000B,00000010B,
	db	01001000B,00000010B,
	db	01001000B,00000010B,
	db	01001000B,00000010B,
	db	01001000B,00000010B,
	db	01001000B,00000010B,
	db	00000100B,00100000B,
	db	00100011B,11000100B,
	db	00010000B,00001000B,
	db	00000111B,11100000B,
	db	00000000B,00000000B,

COPYRIGHT_B:
	db	00000000B,00000000B,
	db	00000000B,00000000B,
	db      01110010B,00001100B,
	db	01001010B,00010010B,
	db      01111010B,00011110B,
	db	01010010B,00010010B,
	db	01001011B,11010010B,
	db	00000000B,00000000B,
	db	00000000B,00000000B,
	db	01001110B,11101110B,
	db	01001010B,10100010B,
	db	01001110B,11100110B,
	db	01000010B,00100010B,
	db	01000010B,00101110B,
	db	00000000B,00000000B,
	db	00000000B,00000000B,

COPYRIGHT_BA:
	db	00000000B,00000000B,
	db	00000000B,00000000B,
	db      01110010B,00001100B,
	db	01001010B,00010010B,
	db      01111010B,00011110B,
	db	01010010B,00010010B,
	db	01001011B,11010010B,
	db	00000000B,00000000B,
	db	00000000B,00000000B,
	db	11101110B,11101010B,
	db	00101010B,10101010B,
	db	01001010B,10101110B,
	db	10001010B,10100010B,
	db	11101110B,11100010B,
	db	00000000B,00000000B,
	db	00000000B,00000000B,

COPYRIGHT_C:
	db	00000000B,00000000B,
	db	00000000B,00000000B,
	db      01110010B,00010000B,
	db	01001010B,00010000B,
	db      01001010B,00010000B,
	db	01001010B,00010000B,
	db	01110011B,11011110B,
	db	00000000B,00000000B,
	db	00000000B,00000000B,
	db	01001110B,11101110B,
	db	01001010B,10100010B,
	db	01001110B,11100110B,
	db	01000010B,00100010B,
	db	01000010B,00101110B,
	db	00000000B,00000000B,
	db	00000000B,00000000B,

COPYRIGHT_CA:
	db	00000000B,00000000B,
	db	00000000B,00000000B,
	db      01110010B,00010000B,
	db	01001010B,00010000B,
	db      01001010B,00010000B,
	db	01001010B,00010000B,
	db	01110011B,11011110B,
	db	00000000B,00000000B,
	db	00000000B,00000000B,
	db	11101110B,11101010B,
	db	00101010B,10101010B,
	db	01001010B,10101110B,
	db	10001010B,10100010B,
	db	11101110B,11100010B,
	db	00000000B,00000000B,
	db	00000000B,00000000B,

PTL0:
	DB	10000000B,00000000B,
	DB	10000000B,00000000B,
	DB	10000000B,00000000B,
	DB	10000000B,00000000B,
	DB	11110111B,00000000B,
	DB	00000010B,00000000B,
	DB	00000010B,00000000B,
	DB	00000010B,01111000B,
	DB	00000111B,01000000B,
	DB	00000000B,01110000B,
	DB	00000000B,01000000B,
	DB	00000000B,01001111B,
	DB	00000000B,00001000B,
	DB	00000000B,00001110B,
	DB	00000000B,00001000B,
	DB	00000000B,00001111B,

$EJECT

;   Copyright message.  Note that this string is _not_ IFDEF DEBUG - the
; copyright string always appears in the ROM, whether or not we actually
; print it!
HLOMSG:	DB	0DH, 0AH, 'LIFE MK5 Copright (C) 1992-2004 by Dan Liddell and Robert Armstrong.', 0DH, 0AH, 0DH, 0AH, 00H

	END



  

