	$TITLE	(RAM Data Segment)


;   This file defines memory map for the 8051 RAM.  It includes the stack,
; the display bitmap, all bit variables, as well as any assorted junk left
; over.

;   REMEMBER!!!!  RAM is not initialized to any special value!  The only
; definitions in this file should be DSs and DBITs.  If you want some
; initial value, you'll have to load it in your code!

;   Locations 0x00 thru 0x1F are used by the four banks of 8 registers.
; These will probably be the first to go when we run out of RAM, but
; for now they are reserved...

;   Locations 0x20 thru 0x2F are the bit addressable segment of RAM.
; This can also be used to contain byte and larger variables, but
; for now we'll leave it alone...

; Copyright (C) 1994 by Dan Liddell and Robert Armstrong.
; Copyright (C) 2004 by Spare Time Gizmos.
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful, but
; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
; ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
; Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, visit the website of the Free
; Software Foundation, Inc., www.gnu.org.

	BSEG AT 020H		; put bit variables here

FLAG_DISPLAYCLOCK: dbit 1; was DISPLAYFLAG. if true, then a display update 
		      ; needed for the clock face. if not, then the clock
		      ; face is fine the way it is now.
FLAG_DISPLAYLIFE: dbit 1 ; if true, then life calls display update. if not,
		      ; then life does not call display update
FLAG_DOFADEUP: dbit 1 ; was DOFADEUP. perform fadeup after display update
FLAG_WHICHBUFF: dbit 1; which walking buffer to use during nextgen computation
		      ; was WHICHBUFF
FLAG_DONTADVANCE: dbit 1 ; if set, then demo mode stays in current pattern 
		      ; until told to advance. If not set, then demo mode
		      ; advances after ten seconds on current pattern
FLAG_DOFADEDOWN: dbit 1 ; perform a fadedown before display update
FLAG_TIMEISSET: dbit 1; user has set the time.
FLAG_DIGITALCLOCK:	dbit 1; use big clock digits
;   The real wide open spaces for general purpose data starts at
; location 0x30 in RAM...
	DSEG AT 028H

BITMAP:		DS	32	; working bitmap for the display
BUFFER0:	DS	2	; left leg of the walking buffer
BUFFER1:	DS	2	; right leg of the walking buffer
BUFFERT:	DS	2	; top to bottom wraparound. row 0

POP1:		DS	1	; most significat byte of population
POP0:		DS 	1	; least significant byte of population

GEN0:		ds	1	; GEN0 - low order of # of gens left to run
GEN1:		ds 	1	; how many generations left to run

MIN0:		DS	1	; current minimum low order byte

LCINDEX:			; current life clock index (uses only runjust)
CYCLE:		DS	1	; # of gens between last minim and this 
LCYCLE:		DS	1	; last # of cycles between identical mimimums

CYCLES:		DS	1	; number of times our minimum count fulfilled

SPEED:		DS	1	; delay between generations

DSECONDS:	DS	1	; the values that are currently displayed
DMINUTES:	DS	1	; on the clock face (any style)
DHOURS:		DS	1

; the reptilian hindbrain, all the autonomic function memory is down here.

IRDATA:		DS	2	; literal 14 bit RC5 code received
IRLAST:		DS	2	; holds the last key code received

INTENSITY:	DS	1	; display intensity (0..F)
BUZZER_TIME:	DS	1	; alarm buzzer timeout value
WAIT_TIME:	DS	1	; timeout for the WAIT routine
POST_CODE:	DS	1	; failure code for power on self test

; TIme of day clock values...
TICKS:		DS	1	; fractional part of a second
ELAPSEDTIME:	DS 	1	; number of seconds since timer started
SECONDS:	DS	1	; seconds, 0..59
MINUTES:	DS	1	; minutes, 0..59
HOURS:		DS	1	; hours (always 24 hour mode!) 0..23

;   The last thing in RAM is always the stack.  Since the 8051 has a
; "bottom up" (i.e. a PUSH instruction _increments_ the stack pointer)
; stack, it goes from the last used RAM location to 0x7F.  We'd better
; have enough!
STACK		EQU	$
STACK_LENGTH	EQU	07FH - STACK
