	$TITLE	(Symbol Definitions)

;   This file contains symbol definitions for the LIFE game.  These include
; LIFE hardware specific values, as well as general assembly parameters.
; No actual code or data is emitted by this module!

; Copyright (C) 1994 by Dan Liddell and Robert Armstrong.
; Copyright (C) 2004 by Spare Time Gizmos.
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful, but
; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
; ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
; Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, visit the website of the Free
; Software Foundation, Inc., www.gnu.org.

; General assembly constants...
BAUD			EQU	0FDH	; serial port baud rate constant
HERTZ			EQU	60	; LTC interrupts per second
BITMAP_SIZE		EQU	32	; currently MUST be 32 (bytes) !!

; MAX7219 interface defintions...
DISPLAY_ECHO		EQU	P1.2	; display data echo (for self test)
DISPLAY_CLOCK		EQU	P1.3	; display data clock
DISPLAY_LOAD		EQU	P1.4	; display command register load
DISPLAY_DATA		EQU	P1.5	; display serial data line

;   These symbols are MAX7219 internal register addresses.  Note that
; registers 1..8 are the data for the corresponding digit!  Refer to the
; Maxim data book for more information.
MAX7219_NOOP		EQU	00H	; no operation (unused) register
MAX7219_MODE		EQU	09H	; decode mode control register
MAX7219_INTENSITY	EQU	0AH	; intensity control register
MAX7219_SCANLIMIT	EQU	0BH	; scan limit register
MAX7219_SHUTDOWN	EQU	0CH	; shutdown control register
MAX7219_TEST		EQU	0DH	; display test register

;   Note that the sense of the BUZZER_BIT is inverted - a HIGH (1) turns
; the buzzer OFF and a LOW (0) turns the buzzer on!  We use two macros,
; BUZZER_ON and BUZZER_OFF, so the code doesn't have to worry about it.
BUZZER_BIT		EQU	P1.0	; controls the piezo beeper
BUZZER_ON	MACRO
	CLR	BUZZER_BIT		; turn the buzzer ON
	ENDM
BUZZER_OFF	MACRO
	SETB	BUZZER_BIT		; turn the buzzer OFF
	ENDM

; Other miscellaneous hardware...
LTC_BIT			EQU	P3.3	; 50/60Hz line time clock (also INT1 !)
IRBIT			EQU	P3.2	; IR receiver data

; Timing constants for a 7.3278Mhz clock for infrared receiver
;RC5HBIT	EQU	543		; one half of a RC5 bit time
;RC5QBIT	EQU	271		;  "  quarter "  "   "    "

; Alternative timing constants for a 11.0592Mhz clock.
RC5HBIT	EQU	819		; one half of a RC5 bit time
RC5QBIT	EQU	410		;  "  quarter "  "   "    "

; And still more alternative constants for a 12.000Mhz clock.
;RC5HBIT	EQU	889		; one half of a RC5 bit time
;RC5QBIT	EQU	445		;  "  quarter "  "   "    "


;   These constants are internal command codes.  The actual keys on the
; verious RC5 remotes are mapped to these command codes by the table at
; RC5KEYS: in module INFRARED.ASM.  The command codes themselves are
; arbitrary and there's not much point in changing them...

; "Modeless" commands which (should!) work anywhere...
CMD_NEXTMODE	EQU	01H	; select the next display mode
CMD_COPYRIGHT	EQU	02H	; show the copyright screen
CMD_BRIGHTNESS	EQU	03H	; change display brightness

; Commands which work only in LIFE modes...
CMD_SHOWMENU	EQU	10H	; display menu
CMD_FASTER	EQU	11H	; go faster
CMD_SLOWER	EQU	12H	; go slower
CMD_NEXTPATTERN	EQU	13H	; next LIFE pattern
CMD_AUTOADVANCE	EQU	14H	; toggle automatic pattern change

;   Commands which work only in clock modes.  These share the same key
; bindings and there fore the same command codes as the LIFE commands.
CMD_STOPCLOCK	EQU CMD_AUTOADVANCE	; stop the clock
CMD_ZEROSECONDS	EQU CMD_SHOWMENU	; reset seconds to zero
CMD_SETSECONDS	EQU CMD_SLOWER		; fast advance the seconds
CMD_SETMINUTES	EQU CMD_FASTER		; fast advance the minutes
CMD_SETHOURS	EQU CMD_NEXTPATTERN	; fast advance the hours
